@@ -205,6 +205,100 @@ class TestLogDevice < Test::Unit::TestCase
     end
   end
 
+  def test_perm_after_shift
+    mode = 0o611
+    File.open(@filename, "w") {|f| f.chmod mode}
+    logfile = @filename
+    logfile0 = logfile + '.0'
+    logdev = d(@filename, shift_age: 1, shift_size: 0)
+    logdev.write('hello')
+    logdev.write('hello')
+    logdev.close
+
+    assert_equal File.stat(logfile0).mode, File.stat(logfile).mode
+  ensure
+    if logfile0
+      File.unlink(logfile0) rescue nil
+    end
+  end
+
+  def test_shifting_size_with_reopen
+    tmpfile = Tempfile.new([File.basename(__FILE__, '.*'), '_1.log'])
+    logfile = tmpfile.path
+    logfile0 = logfile + '.0'
+    logfile1 = logfile + '.1'
+    logfile2 = logfile + '.2'
+    logfile3 = logfile + '.3'
+    tmpfile.close(true)
+    File.unlink(logfile) if File.exist?(logfile)
+    File.unlink(logfile0) if File.exist?(logfile0)
+    File.unlink(logfile1) if File.exist?(logfile1)
+    File.unlink(logfile2) if File.exist?(logfile2)
+
+    logger = Logger.new(STDERR)
+    logger.reopen(logfile, 4, 100)
+
+    logger.error("0" * 15)
+    assert_file.exist?(logfile)
+    assert_file.not_exist?(logfile0)
+    logger.error("0" * 15)
+    assert_file.exist?(logfile0)
+    assert_file.not_exist?(logfile1)
+    logger.error("0" * 15)
+    assert_file.exist?(logfile1)
+    assert_file.not_exist?(logfile2)
+    logger.error("0" * 15)
+    assert_file.exist?(logfile2)
+    assert_file.not_exist?(logfile3)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile3)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile3)
+    logger.close
+    File.unlink(logfile)
+    File.unlink(logfile0)
+    File.unlink(logfile1)
+    File.unlink(logfile2)
+
+    tmpfile = Tempfile.new([File.basename(__FILE__, '.*'), '_2.log'])
+    logfile = tmpfile.path
+    logfile0 = logfile + '.0'
+    logfile1 = logfile + '.1'
+    logfile2 = logfile + '.2'
+    logfile3 = logfile + '.3'
+    tmpfile.close(true)
+    logger = Logger.new(logfile, 4, 150)
+    logger.error("0" * 15)
+    assert_file.exist?(logfile)
+    assert_file.not_exist?(logfile0)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile0)
+    logger.error("0" * 15)
+    assert_file.exist?(logfile0)
+    assert_file.not_exist?(logfile1)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile1)
+    logger.error("0" * 15)
+    assert_file.exist?(logfile1)
+    assert_file.not_exist?(logfile2)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile2)
+    logger.error("0" * 15)
+    assert_file.exist?(logfile2)
+    assert_file.not_exist?(logfile3)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile3)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile3)
+    logger.error("0" * 15)
+    assert_file.not_exist?(logfile3)
+    logger.close
+    File.unlink(logfile)
+    File.unlink(logfile0)
+    File.unlink(logfile1)
+    File.unlink(logfile2)
+  end
+
   def test_shifting_size
     tmpfile = Tempfile.new([File.basename(__FILE__, '.*'), '_1.log'])
     logfile = tmpfile.path
@@ -408,6 +502,12 @@ class TestLogDevice < Test::Unit::TestCase
     end
   end
 
+  def test_open_without_header
+    d(@filename, skip_header: true)
+
+    assert_equal("", File.read(@filename))
+  end
+
   def test_open_logfile_in_multiprocess
     tmpfile = Tempfile.new([File.basename(__FILE__, '.*'), '_1.log'])
     logfile = tmpfile.path
@@ -765,8 +865,6 @@ class TestLogDevice < Test::Unit::TestCase
     end
   end
 
-  env_tz_works = /linux|darwin|freebsd/ =~ RUBY_PLATFORM # borrow from test/ruby/test_time_tz.rb
-
   def test_shifting_weekly_exist_file
     Dir.mktmpdir do |tmpdir|
       assert_in_out_err([{"TZ"=>"UTC"}, *%W"-I#{@top_dir} -rlogger -C#{tmpdir} -"], "#{<<-"begin;"}\n#{<<-'end;'}")
